"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const element_shadow_1 = require("../element-shadow");
class Group extends element_shadow_1.ElementShadow {
    $$ignoreEvalProps = ['runs', 'asyncJobs'];
    runs = [];
    asyncJobs = [];
    init(props) {
        if (Array.isArray(props)) {
            this.runs = props;
        }
        else {
            Object.assign(this, props);
        }
        this.runs = this.runs?.filter(e => e) || [];
        this.asyncJobs = [];
    }
    async createNewElement(nameOrClass, ...args) {
        let elem;
        if (typeof nameOrClass === 'string') {
            const name = nameOrClass;
            const ElemClass = await this.rootScene.tagsManager.loadElementClass(name, this.scene);
            elem = this.scene.elementBuilder
                .element(ElemClass, ...args)
                .tag(name)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        else {
            const name = nameOrClass.name || nameOrClass.constructor?.name;
            const ElemClass = nameOrClass;
            elem = this.scene.elementBuilder
                .element(ElemClass, ...args)
                .tag(name)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        return elem;
    }
    async exec(input) {
        const runs = new Array();
        for (const _e of this.runs) {
            const e = typeof _e === 'string' ? { echo: { content: _e } } : _e;
            const name = Object.keys(e)[0];
            let props = e[name];
            let baseProps = {};
            if (props && typeof props === 'object' && !Array.isArray(props)) {
                let { '<-': inheritKeys, '->': exposeKey, skip, ...eProps } = props;
                if (inheritKeys)
                    eProps = (0, lodash_merge_1.default)({}, this.$$extend(inheritKeys), eProps);
                if (exposeKey)
                    this.$$export(exposeKey, (0, lodash_clonedeep_1.default)(eProps));
                const { if: condition, id, ignoreError, log, vars, async, loop, ...customProps } = eProps;
                baseProps = {
                    id,
                    if: condition,
                    ignoreError,
                    log,
                    vars,
                    async,
                    loop
                };
                props = customProps;
                if (skip)
                    continue;
            }
            const loop = baseProps.loop;
            const condition = baseProps.if;
            const id = baseProps.id;
            if (loop === undefined) {
                if (this.asyncJobs.length && !baseProps?.async) {
                    await Promise.all(this.asyncJobs);
                    this.asyncJobs.splice(0, this.asyncJobs.length);
                }
                if (condition !== undefined) {
                    const isContinue = await this.scene.getVars(condition);
                    if (!isContinue)
                        continue;
                }
                const elem = await this.createNewElement(name);
                if (id)
                    this.scene.localVars[id] = elem;
                Object.assign(elem, baseProps);
                await elem.init((0, lodash_clonedeep_1.default)(props));
                await this.execElement(elem, input);
                runs.push(elem);
            }
            else {
                let loopCondition = await this.scene.getVars(loop, this);
                if (loopCondition) {
                    if (Array.isArray(loopCondition)) {
                        for (let i = 0; i < loopCondition.length; i++) {
                            const loopObj = {
                                loopKey: i,
                                loopValue: loopCondition[i]
                            };
                            if (condition !== undefined) {
                                const isContinue = await this.scene.getVars(condition, loopObj);
                                if (!isContinue)
                                    continue;
                            }
                            const newElem = await this.createNewElement(name);
                            if (id)
                                this.scene.localVars[id] = newElem;
                            Object.assign(newElem, baseProps);
                            Object.assign(newElem, loopObj);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            await this.execElement(newElem, input);
                            runs.push(newElem);
                        }
                    }
                    else if (typeof loopCondition === 'object') {
                        for (const key in loopCondition) {
                            const loopObj = {
                                loopKey: key,
                                loopValue: loopCondition[key]
                            };
                            if (condition !== undefined) {
                                const isContinue = await this.scene.getVars(condition, loopObj);
                                if (!isContinue)
                                    continue;
                            }
                            const newElem = await this.createNewElement(name);
                            if (id)
                                this.scene.localVars[id] = newElem;
                            Object.assign(newElem, baseProps);
                            Object.assign(newElem, loopObj);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            await this.execElement(newElem, input);
                            runs.push(newElem);
                        }
                    }
                    else if (loopCondition === true) {
                        while (loopCondition) {
                            const loopObj = {
                                loopValue: loopCondition
                            };
                            if (condition !== undefined) {
                                const isContinue = await this.scene.getVars(condition, loopObj);
                                if (!isContinue)
                                    continue;
                            }
                            const newElem = await this.createNewElement(name);
                            if (id)
                                this.scene.localVars[id] = newElem;
                            Object.assign(newElem, baseProps);
                            Object.assign(newElem, loopObj);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            await this.execElement(newElem, input);
                            runs.push(newElem);
                            loopCondition = await this.scene.getVars(loop, this);
                        }
                    }
                }
            }
        }
        if (this.asyncJobs.length) {
            await Promise.all(this.asyncJobs);
            this.asyncJobs.splice(0, this.asyncJobs.length);
        }
        return runs;
    }
    async execElement(elem, input) {
        const p = elem.exec(input).finally(() => elem.dispose());
        if (!elem.async) {
            await p;
        }
        else {
            this.asyncJobs.push(p);
        }
    }
}
exports.Group = Group;
//# sourceMappingURL=group.js.map